@@ -27,6 +27,7 @@
 #include <json-glib/json-glib.h>
 #include <libsoup/soup.h>
 #include <geocode-glib/geocode-forward.h>
+#include <geocode-glib/geocode-bounding-box.h>
 #include <geocode-glib/geocode-error.h>
 #include <geocode-glib/geocode-glib-private.h>
 
@@ -43,10 +44,83 @@ struct _GeocodeForwardPrivate {
 	GHashTable *ht;
         SoupSession *soup_session;
 	guint       answer_count;
+	GeocodeBoundingBox *search_area;
+	gboolean bounded;
+};
+
+enum {
+        PROP_0,
+
+        PROP_ANSWER_COUNT,
+        PROP_SEARCH_AREA,
+        PROP_BOUNDED
 };
 
 G_DEFINE_TYPE (GeocodeForward, geocode_forward, G_TYPE_OBJECT)
 
+static void
+geocode_forward_get_property (GObject	 *object,
+			      guint	  property_id,
+			      GValue	 *value,
+			      GParamSpec *pspec)
+{
+	GeocodeForward *forward = GEOCODE_FORWARD (object);
+
+	switch (property_id) {
+		case PROP_ANSWER_COUNT:
+			g_value_set_uint (value,
+					  geocode_forward_get_answer_count (forward));
+			break;
+
+		case PROP_SEARCH_AREA:
+			g_value_set_object (value,
+					    geocode_forward_get_search_area (forward));
+			break;
+
+		case PROP_BOUNDED:
+			g_value_set_boolean (value,
+					     geocode_forward_get_bounded (forward));
+			break;
+
+		default:
+			/* We don't have any other property... */
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+			break;
+	}
+}
+
+static void
+geocode_forward_set_property(GObject	   *object,
+			     guint	    property_id,
+			     const GValue *value,
+			     GParamSpec   *pspec)
+{
+	GeocodeForward *forward = GEOCODE_FORWARD (object);
+
+	switch (property_id) {
+		case PROP_ANSWER_COUNT:
+			geocode_forward_set_answer_count (forward,
+							  g_value_get_uint (value));
+			break;
+
+		case PROP_SEARCH_AREA:
+			geocode_forward_set_search_area (forward,
+							 g_value_get_object (value));
+			break;
+
+		case PROP_BOUNDED:
+			geocode_forward_set_bounded (forward,
+						     g_value_get_boolean (value));
+			break;
+
+		default:
+			/* We don't have any other property... */
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+			break;
+	}
+}
+
+
 static void geocode_forward_add (GeocodeForward *forward,
 				 const char     *key,
 				 const char     *value);
@@ -66,10 +140,60 @@ static void
 geocode_forward_class_init (GeocodeForwardClass *klass)
 {
 	GObjectClass *gforward_class = G_OBJECT_CLASS (klass);
+	GParamSpec *pspec;
 
 	gforward_class->finalize = geocode_forward_finalize;
+	gforward_class->get_property = geocode_forward_get_property;
+	gforward_class->set_property = geocode_forward_set_property;
+
 
 	g_type_class_add_private (klass, sizeof (GeocodeForwardPrivate));
+
+	/**
+	* GeocodeForward:answer-count:
+	*
+	* The number of requested results to a search query.
+	*/
+	pspec = g_param_spec_uint ("answer-count",
+				   "Answer count",
+				   "The number of requested results",
+				   0,
+				   G_MAXINT,
+				   DEFAULT_ANSWER_COUNT,
+				   G_PARAM_READWRITE |
+				   G_PARAM_STATIC_STRINGS);
+	g_object_class_install_property (gforward_class, PROP_ANSWER_COUNT, pspec);
+
+	/**
+	* GeocodeForward:search-area:
+	*
+	* The bounding box that limits the search area.
+	* If #GeocodeForward:bounded property is set to #TRUE only results from
+	* this area is returned.
+	*/
+	pspec = g_param_spec_object ("search-area",
+				     "Search area",
+				     "The area to limit search within",
+				     GEOCODE_TYPE_BOUNDING_BOX,
+				     G_PARAM_READWRITE |
+				     G_PARAM_STATIC_STRINGS);
+	g_object_class_install_property (gforward_class, PROP_SEARCH_AREA, pspec);
+
+	/**
+	* GeocodeForward:bounded:
+	*
+	* If set to #TRUE then only results in the #GeocodeForward:search-area
+	* bounding box are returned.
+	* If set to #FALSE the #GeocodeForward:search-area is treated like a
+	* preferred area for results.
+	*/
+	pspec = g_param_spec_boolean ("bounded",
+				      "Bounded",
+				      "Bind search results to search-area",
+				      FALSE,
+				      G_PARAM_READWRITE |
+				      G_PARAM_STATIC_STRINGS);
+	g_object_class_install_property (gforward_class, PROP_BOUNDED, pspec);
 }
 
 static void
@@ -80,6 +204,8 @@ geocode_forward_init (GeocodeForward *forward)
 						   g_free, g_free);
         forward->priv->soup_session = soup_session_new ();
 	forward->priv->answer_count = DEFAULT_ANSWER_COUNT;
+	forward->priv->search_area = NULL;
+	forward->priv->bounded = FALSE;
 }
 
 static struct {
@@ -343,7 +469,7 @@ get_search_query_for_params (GeocodeForward *forward,
 
 	/* Prepare the query parameters */
 	ht = _geocode_glib_dup_hash_table (forward->priv->ht);
-	g_hash_table_insert (ht, (gpointer) "format", (gpointer) "json");
+	g_hash_table_insert (ht, (gpointer) "format", (gpointer) "jsonv2");
 	g_hash_table_insert (ht, (gpointer) "email", (gpointer) "zeeshanak@gnome.org");
 	g_hash_table_insert (ht, (gpointer) "addressdetails", (gpointer) "1");
 
@@ -364,8 +490,11 @@ get_search_query_for_params (GeocodeForward *forward,
         if (location != NULL) {
 	        /* Prepare the search term */
                 search_term = soup_uri_encode (location, NULL);
-                uri = g_strdup_printf ("http://nominatim.gnome.org/search?q=%s&limit=%u&%s",
-                                       search_term, forward->priv->answer_count, params);
+                uri = g_strdup_printf ("http://nominatim.gnome.org/search?q=%s&limit=%u&bounded=%d&%s",
+                                       search_term,
+                                       forward->priv->answer_count,
+                                       !!forward->priv->bounded,
+                                       params);
                 g_free (search_term);
                 g_free (location);
         } else {
@@ -508,15 +637,6 @@ fill_place_from_entry (const char   *key,
                                       NULL);
                         break;
                 }
-                /* special case for icon, convert to GIcon */
-                if (g_str_equal (key, "icon")) {
-	                GFile *file = g_file_new_for_uri (value);
-	                GIcon *icon = g_file_icon_new (file);
-	                g_object_unref (file);
-
-	                g_object_set (G_OBJECT (place), "icon", icon, NULL);
-	                g_object_unref (icon);
-                }
         }
 }
 
@@ -546,13 +666,13 @@ static const char const *attributes[] = {
 static GeocodePlaceType
 get_place_type_from_attributes (GHashTable *ht)
 {
-        char *class, *type;
+        char *category, *type;
         GeocodePlaceType place_type = GEOCODE_PLACE_TYPE_UNKNOWN;
 
-        class = g_hash_table_lookup (ht, "class");
+        category = g_hash_table_lookup (ht, "category");
         type = g_hash_table_lookup (ht, "type");
 
-        if (g_strcmp0 (class, "place") == 0) {
+        if (g_strcmp0 (category, "place") == 0) {
                 if (g_strcmp0 (type, "house") == 0 ||
                     g_strcmp0 (type, "building") == 0 ||
                     g_strcmp0 (type, "residential") == 0 ||
@@ -594,19 +714,19 @@ get_place_type_from_attributes (GHashTable *ht)
                         place_type =  GEOCODE_PLACE_TYPE_SEA;
                 else if (g_strcmp0 (type, "ocean") == 0)
                         place_type =  GEOCODE_PLACE_TYPE_OCEAN;
-        } else if (g_strcmp0 (class, "highway") == 0) {
+        } else if (g_strcmp0 (category, "highway") == 0) {
                 if (g_strcmp0 (type, "motorway") == 0)
                         place_type =  GEOCODE_PLACE_TYPE_MOTORWAY;
                 else if (g_strcmp0 (type, "bus_stop") == 0)
                         place_type =  GEOCODE_PLACE_TYPE_BUS_STOP;
                 else
                         place_type =  GEOCODE_PLACE_TYPE_STREET;
-        } else if (g_strcmp0 (class, "railway") == 0) {
+        } else if (g_strcmp0 (category, "railway") == 0) {
                 if (g_strcmp0 (type, "station") == 0)
                         place_type =  GEOCODE_PLACE_TYPE_RAILWAY_STATION;
-        } else if (g_strcmp0 (class, "waterway") == 0) {
+        } else if (g_strcmp0 (category, "waterway") == 0) {
                 place_type =  GEOCODE_PLACE_TYPE_DRAINAGE;
-        } else if (g_strcmp0 (class, "boundry") == 0) {
+        } else if (g_strcmp0 (category, "boundary") == 0) {
                 if (g_strcmp0 (type, "administrative") == 0) {
                         int rank;
 
@@ -635,7 +755,7 @@ _geocode_create_place_from_attributes (GHashTable *ht)
 {
         GeocodePlace *place;
         GeocodeLocation *loc = NULL;
-        const char *name, *street, *building;
+        const char *name, *street, *building, *bbox_corner;
         GeocodePlaceType place_type;
         gdouble longitude, latitude;
 
@@ -647,6 +767,28 @@ _geocode_create_place_from_attributes (GHashTable *ht)
 
         place = geocode_place_new (name, place_type);
 
+        /* If one corner exists, then all exists */
+        bbox_corner = g_hash_table_lookup (ht, "boundingbox-top");
+        if (bbox_corner != NULL) {
+            GeocodeBoundingBox *bbox;
+            gdouble top, bottom, left, right;
+
+            top = g_ascii_strtod (bbox_corner, NULL);
+
+            bbox_corner = g_hash_table_lookup (ht, "boundingbox-bottom");
+            bottom = g_ascii_strtod (bbox_corner, NULL);
+
+            bbox_corner = g_hash_table_lookup (ht, "boundingbox-left");
+            left = g_ascii_strtod (bbox_corner, NULL);
+
+            bbox_corner = g_hash_table_lookup (ht, "boundingbox-right");
+            right = g_ascii_strtod (bbox_corner, NULL);
+
+            bbox = geocode_bounding_box_new (top, bottom, left, right);
+            geocode_place_set_bounding_box (place, bbox);
+            g_object_unref (bbox);
+        }
+
         /* Nominatim doesn't give us street addresses as such */
         street = g_hash_table_lookup (ht, "road");
         building = g_hash_table_lookup (ht, "house_number");
@@ -913,6 +1055,105 @@ geocode_forward_set_answer_count (GeocodeForward *forward,
 {
 	g_return_if_fail (GEOCODE_IS_FORWARD (forward));
 
-	/* FIXME: make a property */
 	forward->priv->answer_count = count;
 }
+
+/**
+ * geocode_forward_set_search_area:
+ * @forward: a #GeocodeForward representing a query
+ * @box: a bounding box to limit the search area.
+ *
+ * Sets the area to limit searches within.
+ **/
+void
+geocode_forward_set_search_area (GeocodeForward     *forward,
+				 GeocodeBoundingBox *bbox)
+{
+	char *area;
+	char top[G_ASCII_DTOSTR_BUF_SIZE];
+	char left[G_ASCII_DTOSTR_BUF_SIZE];
+	char bottom[G_ASCII_DTOSTR_BUF_SIZE];
+	char right[G_ASCII_DTOSTR_BUF_SIZE];
+
+	g_return_if_fail (GEOCODE_IS_FORWARD (forward));
+
+	forward->priv->search_area = bbox;
+
+	/* need to convert with g_ascii_dtostr to be locale safe */
+	g_ascii_dtostr (top, G_ASCII_DTOSTR_BUF_SIZE,
+	                geocode_bounding_box_get_top (bbox));
+
+	g_ascii_dtostr (bottom, G_ASCII_DTOSTR_BUF_SIZE,
+	                geocode_bounding_box_get_bottom (bbox));
+
+	g_ascii_dtostr (left, G_ASCII_DTOSTR_BUF_SIZE,
+	                geocode_bounding_box_get_left (bbox));
+
+	g_ascii_dtostr (right, G_ASCII_DTOSTR_BUF_SIZE,
+	                geocode_bounding_box_get_right (bbox));
+
+	area = g_strdup_printf ("%s,%s,%s,%s", left, top, right, bottom);
+	geocode_forward_add (forward, "viewbox", area);
+	g_free (area);
+}
+
+/**
+ * geocode_forward_set_bounded:
+ * @forward: a #GeocodeForward representing a query
+ * @bounded: #TRUE to restrict results to only items contained within the
+ * #GeocodeForward:search-area bounding box.
+ *
+ * Set the #GeocodeForward:bounded property that regulates whether the
+ * #GeocodeForward:search-area property acts restricting or not.
+ **/
+void
+geocode_forward_set_bounded (GeocodeForward *forward,
+			     gboolean        bounded)
+{
+	g_return_if_fail (GEOCODE_IS_FORWARD (forward));
+
+	forward->priv->bounded = bounded;
+}
+
+/**
+ * geocode_forward_get_answer_count:
+ * @forward: a #GeocodeForward representing a query
+ *
+ * Gets the number of requested results for searches.
+ **/
+guint
+geocode_forward_get_answer_count (GeocodeForward *forward)
+{
+	g_return_val_if_fail (GEOCODE_IS_FORWARD (forward), 0);
+
+	return forward->priv->answer_count;
+}
+
+/**
+ * geocode_forward_get_search_area:
+ * @forward: a #GeocodeForward representing a query
+ *
+ * Gets the area to limit searches within.
+ **/
+GeocodeBoundingBox *
+geocode_forward_get_search_area (GeocodeForward *forward)
+{
+	g_return_val_if_fail (GEOCODE_IS_FORWARD (forward), NULL);
+
+	return forward->priv->search_area;
+}
+
+/**
+ * geocode_forward_get_bounded:
+ * @forward: a #GeocodeForward representing a query
+ *
+ * Gets the #GeocodeForward:bounded property that regulates whether the
+ * #GeocodeForward:search-area property acts restricting or not.
+ **/
+gboolean
+geocode_forward_get_bounded (GeocodeForward *forward)
+{
+	g_return_val_if_fail (GEOCODE_IS_FORWARD (forward), FALSE);
+
+	return forward->priv->bounded;
+}
